﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace WorldMapKitConverter
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {

        }

        private void btnConvert_Click(object sender, EventArgs e)
        {
            // We want to split out individual states from the TLKA, WAT, LPT, and CTYCU folders
            ConvertFolder(txtSourceFolder.Text, "TLKA");
            ConvertFolder(txtSourceFolder.Text, "WAT");
            ConvertFolder(txtSourceFolder.Text, "LPT");
            ConvertFolder(txtSourceFolder.Text, "CTYCU");
        }

        /// <summary>
        /// Split out the individual state files from one larger folder that
        /// contains all states
        /// </summary>
        /// <param name="sourceFolder">The path to the full source data folder</param>
        /// <param name="folder">The name of the sub folder from which to extract
        /// individual state files</param>
        private void ConvertFolder(string sourceFolder, string folder)
        {
            string subFolder = Path.Combine(Path.Combine(sourceFolder, "USA"), folder);
            string[] shapefiles = Directory.GetFiles(subFolder, "*.shp");

            // Iterate every shapefile and move it into an appropriate state-specific subfolder, 
            // and then build an index for it. 
            foreach (string shapefile in shapefiles)
            {
                string state = Path.GetFileName(shapefile).Substring(0, 2);
                string stateFolder = Path.Combine(subFolder, state);
                Directory.CreateDirectory(stateFolder);

                // A little progress info for those who are impatient
                txtCurrent.Text = shapefile;
                txtCurrent.Refresh();

                MoveShapefile(shapefile, stateFolder);
            }
        }

        /// <summary>
        /// Move a shapefile into a different folder, and then build an index for it.
        /// </summary>
        /// <param name="shapefile"></param>
        /// <param name="targetFolder"></param>
        private void MoveShapefile(string shapefile, string targetFolder)
        {
            string filename = shapefile;
            string newFilename = Path.Combine(targetFolder, Path.GetFileName(shapefile));
            File.Move(filename, newFilename );
            filename = Path.ChangeExtension(filename, ".shx");
            File.Move(filename, Path.Combine(targetFolder, Path.GetFileName(filename)));
            filename = Path.ChangeExtension(filename, ".dbf");
            File.Move(filename, Path.Combine(targetFolder, Path.GetFileName(filename)));

            try
            {
                ThinkGeo.MapSuite.Core.ShapeFileFeatureLayer.BuildIndexFile(newFilename);
            }
            catch
            {
                //Ignore index file errors.   Errors are generated when indexing a shapefile with no records.
            }
        }

        /// <summary>
        /// This command is used to build install scripts for the Advanced Installer package.  The goal is to
        /// build a script for each state.  For Advanced Installer, we built the first state manually and used
        /// it as a template.  Then we simply replaced GUIDs and wild cards with info to make each state's
        /// installer file unique.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnBuildInstalls_Click(object sender, EventArgs e)
        {
            string batch = "";

            // Iterate through all of the states folders
            string lptFolder = Path.Combine(Path.Combine(txtSourceFolder.Text, "USA"), "LPT");
            string[] states = Directory.GetDirectories(lptFolder);

            // Load the template install file into a string
            string installFolder = Path.Combine(txtSourceFolder.Text, "Installs");
            string install = File.ReadAllText(Path.Combine(installFolder, "State_ReferenceData_Template.aip"));

            // Iterate through each state to update the template with state specific information.
            foreach (string state in states)
            {
                string abbr = Path.GetFileName(state);
                // Replace the state abbreviation in key spots
                string t = install.Replace("??", abbr);

                // Replace the Guids used to define the install package.
                t = t.Replace("10B3DC66-5430-4194-B082-BDCB3F07A6E2", Guid.NewGuid().ToString());
                t = t.Replace("B465C207-F0BB-490A-9995-168D6374394E", Guid.NewGuid().ToString());
                t = t.Replace("E9E93D42-90A2-4ED7-A2B0-26CA83B98F4B", Guid.NewGuid().ToString());
                t = t.Replace("EBF1A9AB-EFA4-4AA2-B8BE-DCF40AC88E75", Guid.NewGuid().ToString());
                t = t.Replace("9F8ECBEE-8DB6-4016-9853-6E4315EE7E26", Guid.NewGuid().ToString());
                t = t.Replace("AD43DE35-A618-4D58-B974-05172C3ABDF0", Guid.NewGuid().ToString());
                t = t.Replace("2616FBBD-433A-42D7-9417-101C64883D08", Guid.NewGuid().ToString());

                // Write the modified template as a state-specific install script
                string newFile = abbr + "_ReferenceData.aip";
                File.WriteAllText(Path.Combine(installFolder, newFile), t);

                // Accumulate a batch file command to build all installs.
                batch = batch + "\"C:\\Program Files\\Caphyon\\Advanced Installer 7.1.1\\AdvancedInstaller.Com\"" +
                                " /build \"" + newFile + "\"\r\n";

            }

            File.WriteAllText(Path.Combine(installFolder, "_build.bat"), batch);
        }
    }
}
